#!/usr/bin/env python
# coding: utf-8

from builtins import *

import marcx
import xlrd

# Default input and output.
inputfilename = "99 Media Perspektiven Aug 2017.xlsx"
outputfilename = "99_output.mrc"

if len(sys.argv) == 3:
	inputfilename, outputfilename = sys.argv[1:]

outputfile = open(outputfilename, "wb")

workbook = xlrd.open_workbook(inputfilename)
sheet = workbook.sheet_by_name("Tabelle3")

for i, row in enumerate(range(sheet.nrows), start=0):
	
	csv_record = sheet.row_values(row)
	marc_record = marcx.Record(force_utf8=True)

	if csv_record[0] == "authors":
		continue

	# Leader
	marc_record.leader = "     naa  22        4500"

	# ID
	f001 = str(i)
	marc_record.add("001", data="finc-99-" + f001)
	
	# 007
	marc_record.add("007", data="cr")

	#008
	f260c = csv_record[8]
	f260c = str(f260c).rstrip(".0")
	if len(f260c) == 4:
		marc_record.add("008", data="130227u%suuuuuxx uuup     c" % f260c)
	else:
		marc_record.add("008", data="130227uu20uuuuuuxx uuup     c")

	# ISSN
	f022a = csv_record[5]
	marc_record.add("022", a=f022a)

	# 1. Schöpfer
	f100a = csv_record[0]
	f100a = f100a.split("; ")
	f100a = f100a[0]
	marc_record.strict = False
	marc_record.add("100", a=f100a)
	marc_record.strict = True

	# Haupttitel
	f245a = csv_record[1]
	f245a = str(f245a)
	marc_record.add("245", a=f245a)

	# Verlag, Erscheinungsjahr
	f260b = csv_record[3]
	f260b = str(f260b)
	f260b = f260b.lstrip("Frankfurt: ")
	f260c = csv_record[8]
	f260c = str(f260c).rstrip(".0")
	publisher = ["a", "Frankfurt : ", "b", f260b + ", ", "c", f260c]
	marc_record.add("260", subfields=publisher)
	
	# Seitenzahl
	f300a = csv_record[11]
	f300a = str(f300a).rstrip(".0")
	marc_record.add("300", a=f300a)

	# weitere Schöpfer
	f700a = csv_record[0]
	f700a = f700a.split("; ")
	if len(f700a) > 1:
		for person in f700a[1:]:
			marc_record.add("700", a=person)

	# Quelle
	f773t = csv_record[2]
	f773t = str(f773t)
	issue = csv_record[6]
	issue = str(issue).rstrip(".0")
	volume = csv_record[7]
	volume = str(volume).rstrip(".0")
	year = csv_record[8]
	year = str(year).rstrip(".0")
	pages = csv_record[12]
	pagess = str(pages).rstrip(".0")	
	f773g = "%s(%s)%s, S. %s" % (volume, year, issue, pages)
	marc_record.add("773", g=f773g, t=f773t)

	# Link zur Ressource
	f856u = csv_record[13]
	marc_record.add("856", q="text/html", _3="Link zur Ressource", u=f856u)

	# Medienform
	marc_record.add("935", b="cofz")

	# Kollektion
	marc_record.add("980", a=f001, b="99", c="Media Perspektiven")


	outputfile.write(marc_record.as_marc())

outputfile.close()



# coding: utf-8
# pylint: disable=C0301

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""

Support for importing all sources at once. See also:
http://stackoverflow.com/a/1057534/89391

"""

import glob
import os

modules = glob.glob("%s/*.py" % (os.path.dirname(__file__)))
__all__ = [os.path.basename(f)[:-3] for f in modules]



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Datenbank Internetquellen, #5374, #9913, #11005.

Fixed source from a single file from a defunct http://www.medien-buehne-film.de/.

Configuration
-------------

[dbinet]

directory = /path/to/dir/containing/raw/files

"""

import glob
import json
import os
import tempfile

import luigi
import requests
from gluish.format import TSV
from gluish.utils import shellout

from siskin.task import DefaultTask


class DBInetTask(DefaultTask):
    """ Base task. """
    TAG = "80"


class DBInetFiles(DBInetTask):
    """
    A list of raw files.
    """

    def run(self):
        directory = self.config.get('dbinet', 'directory')
        with self.output().open('w') as output:
            for path in sorted(glob.glob(os.path.join(directory, "*xml.part*"))):
                output.write_tsv(path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='filelist'), format=TSV)


class DBInetJSON(DBInetTask):
    """
    Combine, prepare.
    """

    def requires(self):
        return DBInetFiles()

    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path',)):
                shellout("""python {xmltojson} <(xsltproc {xsl} <(sed -e 's/&#.;//g' -e 's/&#..;//g' {input})) /dev/stdout |
                            jq -cr ".list[][]" >> {output} """,
                         xmltojson=self.assets('80/xmltojson.py'),
                         xsl=self.assets('80/convert.xsl'), input=row.path, output=stopover)

        luigi.LocalTarget(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))


class DBInetIntermediateSchema(DBInetTask):
    """ Convert via jq. Filter out dead links (takes a while). """

    urlcheck = luigi.BoolParameter(default=False, significant=False)

    def requires(self):
        return DBInetJSON()

    def run(self):
        """ When URLs are checked, only write records with at least one reachable URL. """
        output = shellout("jq -rc -f {filter} {input} > {output}",
                          filter=self.assets('80/filter.jq'), input=self.input().path)

        if not self.urlcheck:
            luigi.LocalTarget(output).move(self.output().path)
            return

        with luigi.LocalTarget(output).open() as handle:
            with self.output().open('w') as output:
                for line in handle:
                    doc = json.loads(line)
                    okurls = []
                    for url in doc.get("url"):
                        try:
                            resp = requests.get(url, timeout=20)
                            self.logger.debug("[%s] %s", resp.status_code, url)
                            if resp.status_code >= 400:
                                continue
                            okurls.append(url)
                        except (requests.exceptions.ConnectionError,
                                requests.exceptions.ReadTimeout,
                                requests.exceptions.TooManyRedirects) as err:
                            self.logger.debug(err)
                            continue

                    if len(okurls) == 0:
                        continue

                    doc["urls"] = okurls
                    output.write(json.dumps(doc))
                    output.write("\n")

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))



# coding: utf-8
# pylint: disable=C0103,C0411,F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
#

"""
VKFilm (BW), #8700, about 40000 records, Alephino.

TODO: Unify VK* tasks.

Config
------

[vkbw]

baseurl = http://alephino.club/x
username = ADMIN
password = SECRET
"""

from __future__ import print_function

import datetime
import io
import urllib
from xml.dom.minidom import parseString

import luigi
import requests
import xmltodict
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter

from siskin.task import DefaultTask


class VKBWTask(DefaultTask):
    """
    Base task for VKFilm BW.
    """
    TAG = '151'

    def closest(self):
        return monthly(date=self.date)


class VKBWDownload(VKBWTask):
    """
    Harvest MARCXML-ish records from Alephino.
    """

    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        baseurl = self.config.get('vkbw', 'baseurl')
        query = {
            'op': 'find',
            'base': 'B-TIT',
            'query': 'IDN=1 < *',
            'usr': self.config.get('vkbw', 'username'),
            'pwd': self.config.get('vkbw', 'password'),
        }
        url = "%s?%s" % (baseurl, urllib.urlencode(query))

        r = requests.get(url)
        if r.status_code != 200:
            raise RuntimeError('got HTTP %s: %s' % (r.status_code, url))

        dd = xmltodict.parse(r.text)
        set_number = int(dd["find"]["set_number"])
        no_entries = int(dd["find"]["no_entries"])

        with self.output().open('w') as output:
            output.write(u"""<?xml version="1.0" encoding="UTF-8" ?>""")
            output.write(u"<collection>")
            for no in range(1, no_entries + 1):
                query = {
                    'op': 'getrec',
                    'set_number': set_number,
                    'number_entry': no,
                    'usr': self.config.get('vkbw', 'username'),
                    'pwd': self.config.get('vkbw', 'password'),
                }
                url = "%s?%s" % (baseurl, urllib.urlencode(query))
                resp = requests.get(url)
                if resp.status_code != 200:
                    print('failed with HTTP %s: %s' % (resp.status_code, url))
                    continue

                doc = parseString(resp.text.encode('utf-8').strip())
                document = doc.childNodes[0].childNodes[0]
                document.tagName = 'record'
                document.removeAttribute('base')
                document.removeAttribute('idn')
                output.write(document.toxml("utf-8"))

                self.logger.debug("fetched %s/%s", no, no_entries)
            output.write(u"</collection>")

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))



# coding: utf-8
# pylint: disable=C0301,C0103

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Decorators used in siskin.
"""

import functools
import warnings


def deprecated(func):
    """
    Mark functions as deprecated. It will result in a warning being emmitted
    when the function is used.

    TODO: emit a bit more useful text.
    """

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        warnings.simplefilter('always', DeprecationWarning)  # turn off filter
        warnings.warn("call to deprecated function {}.".format(func.__name__),
                      category=DeprecationWarning, stacklevel=2)
        warnings.simplefilter('default', DeprecationWarning)  # reset filter
        return func(*args, **kwargs)

    return new_func



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Example workflow with OAI harvest and metafacture.
"""

import datetime

import luigi
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class MHLibraryTask(DefaultTask):
    """ Base task for MH Library """
    TAG = '103'

    def closest(self):
        return monthly(date=self.date)


class MHLibraryHarvest(MHLibraryTask):
    """
    Harvest.
    """
    endpoint = luigi.Parameter(
        default='http://cdm15759.contentdm.oclc.org/oai/oai.php', significant=False)
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        shellout("""metha-sync "{endpoint}" """, endpoint=self.endpoint)
        output = shellout(
            """metha-cat -root collection "{endpoint}" > {output}""", endpoint=self.endpoint)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class MHLibraryMARC(MHLibraryTask):
    """
    Convert OAI DC to MARCXML via custom Python script (TBR).
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return MHLibraryHarvest(date=self.date)

    def run(self):
        output = shellout("python {script} {input} {output}",
                          script=self.assets('103/103_marcbinary.py'),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))

    def on_success(self):
        """
        Signal for manual update. This is decoupled from the task output,
        because it might be deleted, when imported.
        """
        self.output().copy(self.path(ext='fincmarc.mrc.import'))



#!/usr/bin/env python
# coding: utf-8

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
siskin is a set of tasks for library metadata management.
"""

from __future__ import print_function

import sys

from siskin import __version__

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


install_requires = [
    'astroid>=1.1.1',
    'backoff>=1.3.1',
    'beautifulsoup4',
    'colorama>=0.3.3',
    'configparser>=3.5.0',
    'csvkit==0.9.1',
    'decorator>=3.4.0',
    'elasticsearch>=2',
    'future>=0.16',
    'gluish>=0.2.10',
    'gspread>=0.2.2',
    'internetarchive>=1.7.6',
    'jsonpath-rw>=1.3.0',
    'logilab-common>=0.61.0',
    'luigi>=2.2',
    'lxml>=3.4.2',
    'marcx>=0.2.9',
    'nose>=1.3.3',
    'ply>=3.4',
    'prettytable>=0.7.2',
    'protobuf>=2.6.1',
    'pygments>=2',
    'pyisbn>=1.0.0',
    'pylint>=1.2.1',
    'pymarc>=3.0.1',
    'pymysql>=0.7',
    'pyparsing>=2.0.3',
    'python-dateutil==2.2',
    'pytz>=2014.4',
    'rdflib>=4',
    'requests>=2.5.1',
    'simplejson>=3.6.0',
    'six>=1.9.0',
    'snakebite>=2.5.1',
    'tqdm>=4',
    'ujson>=1.35',
    'urllib3>=1.10',
    'xlrd==1.0.0',
    'XlsxWriter>=1.0.2',
    'xmltodict==0.11.0',
]

if sys.version_info.major < 3:
    install_requires += ['argparse>=1.2', 'wsgiref>=0.1.2']


print("""
Installation note:

    If you see an error like

    > AttributeError: 'ChangelogAwareDistribution' object has no attribute '_egg_fetcher'

    or similar from python-daemon, it's unfortunate but fixable: install
    python-daemon manually, then run setup.py again:

    $ pip install python-daemon
    $ python setup.py develop

--------
""")

setup(name='siskin',
      version=__version__,
      description='Various sources and workflows.',
      url='https://github.com/ubleipzig/siskin',
      author='The Finc Authors',
      author_email='team@finc.info',
      packages=[
          'siskin',
          'siskin.sources',
          'siskin.workflows'
      ],
      package_dir={'siskin': 'siskin'},
      package_data={
          'siskin': [
              'assets/*',
              'assets/1/*',
              'assets/10/*',
              'assets/100/*',
              'assets/101/*',
              'assets/103/*',
              'assets/107/*',
              'assets/117/*',
              'assets/124/*',
              'assets/127/*',
              'assets/143/*',
              'assets/148/*',
              'assets/15/*',
              'assets/150/*',
              'assets/153/*',
              'assets/30/*',
              'assets/34/*',
              'assets/39/*',
              'assets/52/*',
              'assets/56_57_58/*',
              'assets/73/*',
              'assets/78/*',
              'assets/80/*',
              'assets/87/*',
              'assets/88/*',
              'assets/99/*',
              'assets/156/*',
              'assets/arxiv/*',
              'assets/datacite/*',
              'assets/js/*',
              'assets/maps/*',
              'assets/oai/*',
          ]},
      scripts=[
          'bin/taskcat',
          'bin/taskchecksetup',
          'bin/taskconfig',
          'bin/taskdeps-dot',
          'bin/taskdir',
          'bin/taskdo',
          'bin/taskdocs',
          'bin/taskdu',
          'bin/taskhash',
          'bin/taskhead',
          'bin/taskhelp',
          'bin/taskhome',
          'bin/taskimportcache',
          'bin/taskinspect',
          'bin/taskless',
          'bin/taskls',
          'bin/tasknames',
          'bin/taskopen',
          'bin/taskoutput',
          'bin/taskps',
          'bin/taskredo',
          'bin/taskrm',
          'bin/taskstatus',
          'bin/tasktree',
          'bin/taskversion',
          'bin/taskwc',
          'bin/xmltools.py',
      ],
      install_requires=install_requires,
      zip_safe=False,
      classifier=[
          'Development Status :: 4 - Beta',
          'Environment :: Console'
          'License :: GPLv3',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Topic :: Text Processing',
      ])



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Disson, Dissertations Online, refs #3422.
"""

import datetime

import luigi

from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class DissonTask(DefaultTask):
    """
    Base task for disson.
    """
    TAG = '13'

    def closest(self):
        return monthly(date=self.date)


class DissonHarvest(DissonTask):
    """
    Via OAI, access might be restricted.

    ;; ANSWER SECTION:
    services.dnb.de.	3539	IN	CNAME	httpd-e02.dnb.de.
    httpd-e02.dnb.de.	3539	IN	CNAME	prodche-02.dnb.de.
    prodche-02.dnb.de.	3539	IN	A	    193.175.100.222

    Format options (2018-03-02):

    * baseDc-xml
    * BibframeRDFxml
    * JATS-xml
    * MARC21plus-1-xml
    * MARC21plus-xml
    * MARC21-xml
    * mods-xml
    * oai_dc
    * ONIX-xml
    * PicaPlus-xml
    * RDFxml
    * sync-repo-xml
    * xMetaDiss

    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default='MARC21-xml')
    set = luigi.Parameter(default='dnb-all:online:dissertations')

    def run(self):
        endpoint = "http://services.dnb.de/oai/repository"
        shellout("metha-sync -daily -format {format} -set {set} {endpoint}",
                 format=self.format, set=self.set, endpoint=endpoint)
        output = shellout("""metha-cat -format {format} -set {set} {endpoint} |
                             pigz -c > {output}""", format=self.format, set=self.set,
                          endpoint=endpoint)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml.gz', digest=True))


class DissonMARC(DissonTask):
    """
    Create a binary MARC version.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DissonHarvest(date=self.date)

    def run(self):
        output = shellout("yaz-marcdump -i marcxml -o marc <(unpigz -c {input}) > {output}",
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))
